<?php
/**
 * WooCommerce Compatibility File
 *
 * @link https://woocommerce.com/
 *
 * @package YITH_Booking
 */

/**
 * WooCommerce setup function.
 *
 * @link https://docs.woocommerce.com/document/third-party-custom-theme-compatibility/
 * @link https://github.com/woocommerce/woocommerce/wiki/Enabling-product-gallery-features-(zoom,-swipe,-lightbox)-in-3.0.0
 *
 * @return void
 */
function yith_booking_woocommerce_setup() {
    add_theme_support( 'woocommerce' );
    add_theme_support( 'wc-product-gallery-lightbox' );

    if ( apply_filters( 'yith_booking_woocommerce_gallery_in_header', true ) ) {
        remove_action( 'woocommerce_before_single_product_summary', 'woocommerce_show_product_sale_flash', 10 );
        remove_action( 'woocommerce_before_single_product_summary', 'woocommerce_show_product_images', 20 );
        add_action( 'yith_booking_content_header', 'yith_booking_wc_image_gallery_in_header_content' );
        add_filter( 'body_class', 'yith_booking_add_body_class_for_wc_image_gallery', 10, 1 );
        add_filter( 'woocommerce_gallery_image_size', 'yith_booking_woocommerce_gallery_image_size_full', 999 );
    } else {
        add_theme_support( 'wc-product-gallery-zoom' );
        add_theme_support( 'wc-product-gallery-slider' );
    }
}

add_action( 'after_setup_theme', 'yith_booking_woocommerce_setup' );

/**
 * Register widget area.
 *
 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
 */
function yith_booking_woocommerce_product_widgets_init() {
    register_sidebar( array(
                          'name'          => esc_html__( 'Product Sidebar', 'yith-booking' ),
                          'id'            => 'sidebar-product',
                          'description'   => esc_html__( 'Add widgets here.', 'yith-booking' ),
                          'before_widget' => '<section id="%1$s" class="widget %2$s">',
                          'after_widget'  => '</section>',
                          'before_title'  => '<h2 class="widget-title">',
                          'after_title'   => '</h2>',
                      ) );
}

add_action( 'widgets_init', 'yith_booking_woocommerce_product_widgets_init' );


/**
 * WooCommerce specific scripts & stylesheets.
 *
 * @return void
 */
function yith_booking_woocommerce_scripts() {
    wp_enqueue_style( 'yith-booking-woocommerce-style', get_template_directory_uri() . '/woocommerce.css' );
    wp_enqueue_script( 'yith-booking-woocommerce-script', get_template_directory_uri() . '/js/woocommerce.js', array( 'jquery' ), '1.0.0', true );

    $font_path = WC()->plugin_url() . '/assets/fonts/';
    $inline_font
               = '@font-face {
			font-family: "star";
			src: url("' . $font_path . 'star.eot");
			src: url("' . $font_path . 'star.eot?#iefix") format("embedded-opentype"),
				url("' . $font_path . 'star.woff") format("woff"),
				url("' . $font_path . 'star.ttf") format("truetype"),
				url("' . $font_path . 'star.svg#star") format("svg");
			font-weight: normal;
			font-style: normal;
		}';

    wp_add_inline_style( 'yith-booking-woocommerce-style', $inline_font );
}

add_action( 'wp_enqueue_scripts', 'yith_booking_woocommerce_scripts' );

function yith_booking_wc_image_gallery_in_header_content() {
    if ( function_exists( 'is_product' ) && is_product() ) {
        global $product;
        $product = wc_get_product();
        woocommerce_show_product_sale_flash();
        echo "<div class='yith-booking-woocommerce-images'>";
        woocommerce_show_product_images();
        $attachment_ids = $product->get_gallery_image_ids();
        if ( $attachment_ids ) {
            echo "<span class='open-gallery'>" . __( 'Show Images', 'yith-booking' ) . "</span>";
        }
        echo '</div>';
    }
}

function yith_booking_add_body_class_for_wc_image_gallery( $classes ) {
    $classes[] = 'yith-booking-has-woocommerce-gallery-in-header';

    return $classes;
}

function yith_booking_woocommerce_gallery_image_size_full() {
    return 'full';
}

